///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp Programming Examples
//
// Copyright (C) 2012 Tieto Sweden AB - All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//		TF	Dec 31, 2012	Created
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.facet;

import com.tietoenator.trip.jxp.TdbException;
import com.tietoenator.trip.jxp.TdbMessageProvider;
import com.tietoenator.trip.jxp.examples.util.Console;
import com.tietoenator.trip.jxp.examples.util.SessionFactory;
import com.tietoenator.trip.jxp.facet.TdbClassificationFacet;
import com.tietoenator.trip.jxp.facet.TdbFacet;
import com.tietoenator.trip.jxp.facet.TdbFacetSet;
import com.tietoenator.trip.jxp.facet.TdbFacetSortOrder;
import com.tietoenator.trip.jxp.facet.TdbFacetValue;
import com.tietoenator.trip.jxp.facet.TdbFieldTermFacet;
import com.tietoenator.trip.jxp.facet.TdbKvpFacet;
import com.tietoenator.trip.jxp.facet.TdbSearchFacet;
import com.tietoenator.trip.jxp.session.TdbSession;

/**
 * Example of how to use the facet classes in TRIPjxp.
 */
public class FacetSet
{
    private TdbSession m_session;
    
    public FacetSet()
            throws TdbException
    {
        // Create a session on a physical server (no grid support)
        m_session = SessionFactory.newInstance("**** TRIP Facet Example ****", false);
    }
    
    public void close()
    {
        if (m_session != null)
        {
            
            try
            {
                m_session.logout();
            }
            catch (TdbException ignore)
            {
            }
            finally
            {
                m_session = null;
            }
        }
    }


   // Show any success message that might be forthcoming from the server

   private static void showMessages(TdbMessageProvider prv)
   {
      for (String msg : prv.messageList())
      {
         System.out.println(msg);
      }
      prv.resetMessages();
   }
   
   
   private void printFacetValues(TdbFacet f)
   {
       System.out.println();
       System.out.println(f.getName()+":");
       for (TdbFacetValue v : f)
       {
           System.out.print("["+Integer.toString(v.getRecordCount())+"]\t");
           System.out.println(v.toString());
       }
   }
   
   private String readnoblank(Console stdin, String prefix, String defaultValue)
           throws TdbException
   {
       String inval;
       do
       {
           inval = stdin.readln(prefix,defaultValue,true);
       }
       while (inval.length()==0);
       return inval;
       
   }
   
   
   public void process(Console stdin) throws TdbException
   {
       TdbFacetSet facetset;
       TdbFacet facet = null;
       int facetIdx;
       int facetType;
       int sortType;
       String facetDb;
       String facetId;
       String facetNameField;
       String facetValueField;
       String facetValueMask;
       String facetQuery;
       String facetValueId;
       boolean morevalues;
       int lowerbound;
       int upperbound;
       
       // The TdbFacetSet is our container facets and also a dispatcher 
       // facet requests to the server.
       facetset = new TdbFacetSet(m_session);

       // Specify facets by asking the user for parameters.
       do
       { 
           System.out.println();
           do
           {
               facetType = stdin.readint("Facet type (1=TermList,2=KVP,3=Class,4=Search)", 1, true);
           }
           while (facetType<1 || facetType>4);
           
           facetDb = readnoblank(stdin,"Name of facet database","");
           
           switch (facetType)
           {
               case 1:
                   facetValueField = readnoblank(stdin,"Field to base facet upon","");
                   facetValueMask = readnoblank(stdin,"Facet value search mask","#");
                   facet = new TdbFieldTermFacet(facetValueField, facetValueMask);
                   break;
               case 2:
                   facetNameField = readnoblank(stdin,"Field with facet names","");
                   facetId = readnoblank(stdin,"Facet name (value from the names field)","");
                   facetValueField = readnoblank(stdin,"Field with facet values","");
                   facetValueMask = readnoblank(stdin,"Facet value search mask","#");
                   facet = new TdbKvpFacet(facetNameField, facetValueField, facetId, facetValueMask);
                   break;
               case 3:
                   facetId = readnoblank(stdin,"Facet ID string","");
                   facetValueMask = readnoblank(stdin,"Facet value search mask","#");
                   facet = new TdbClassificationFacet(facetId, facetValueMask);
                   break;
               case 4:
                   facetId = readnoblank(stdin,"Facet ID string","");
                   TdbSearchFacet searchFacet = new TdbSearchFacet(facetId);
                   
                   System.out.println();
                   System.out.println(">> A search facet has facet values each of which correspond to a search result.");
                   System.out.println(">> The id for each facet value and its corresponding FIND/FUZZ CCL command must");
                   System.out.println(">> be specified.");
                   System.out.println();

                   do
                   {
                       facetValueId = readnoblank(stdin,"Facet value ID","");
                       facetQuery = readnoblank(stdin,"CCL query (incl FIND/FUZZ command)","");
                       searchFacet.add(facetValueId, facetQuery);
                       morevalues = stdin.readbool("More values?", true, true);
                   }
                   while (morevalues);
                   
                   facet = searchFacet;
                   break;
           }
           
           facet.setDatabase(facetDb);
           
           if (facetType != 4 && stdin.readbool("Specify sort order?", false, true))
           {
               System.out.println();
               System.out.println("Which sort order to you wish to use?");
               System.out.println("1) Sort by frequency, highest frequency facet value first");
               System.out.println("2) Sort by frequency, lowest frequency facet value first");
               System.out.println("3) Sort by value, reverse alphabetical order");
               System.out.println("4) Sort by value, alphabetical order");
               System.out.println();
               
               do
               {
                   sortType = stdin.readint("Please specify",1,false);
               }
               while (sortType<1 || sortType>4);
               System.out.println();
               
               switch (sortType)
               {
                   case 1:
                       facet.setSortOrder(TdbFacetSortOrder.FrequencyDescending); break;
                   case 2:
                       facet.setSortOrder(TdbFacetSortOrder.FrequencyAscending); break;
                   case 3:
                       facet.setSortOrder(TdbFacetSortOrder.ValueDescending); break;
                   case 4:
                       facet.setSortOrder(TdbFacetSortOrder.ValueAscending); break;
               }
           }
           
           if (stdin.readbool("Specify range of values to retrieve?", true, true))
           {
               lowerbound = stdin.readint("Lowest ordinal number of values", 1, true);
               upperbound = stdin.readint("Highest ordinal number of values", 100, true);
               
               facet.setFetchFrom(lowerbound);
               facet.setFetchTo(upperbound);
           }
           
           if (facetType != 4 && stdin.readbool("Specify frequency limits?", false, true))
           {
               lowerbound = stdin.readint("Lowest occurrence frequency", 1, true);
               upperbound = stdin.readint("Highest occurrence frequency (0=no limit)", 0, true);
               facet.setFrequencyLimits(lowerbound, upperbound);
           }

           facetset.add(facet);
           System.out.println();
       }
       while (stdin.readbool("Additional facets?", false, true));
       
       // Retrieve values for all facets now
       facetset.get();
       
       // Use facet set. Here, only print out the facet values.
       for (facetIdx=0;
            facetIdx<facetset.getFacetCount();
            facetIdx++)
       {
           printFacetValues(facetset.getFacet(facetIdx));
       }
       
       // Remember to close the facet set immediately when you're done with it.
       // Don't allow garbage collection to close it for you, since the facet
       // set may have server-side resources associated with it!
       facetset.close();
   }
   
   
   public static void main(String[] args)
   {
       FacetSet app = null;
       Console stdin = new Console();
       try
       {
           app = new FacetSet();
           app.process(stdin);
       }
       catch (TdbException tx)
       {
           System.err.println(tx.getDebugMessage());
       }
       catch (Exception ox)
       {
           ox.printStackTrace(System.err);
       }
       finally
       {
           if (app != null)
               app.close();
       }
       
       
   }
}
